
# q-expansion of newform 882.6.a.t, downloaded from the LMFDB on 03 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[4], [0], [26], [0], [358], [-332], [126], [2200], [2142], [3610], [-5668], [-2922], [-2142], [6388], [-6520], [10702], [42524], [44840], [-1448], [4402], [-20500], [65236], [-102804], [-128006], [113324], [-139714], [142180], [198518], [132538], [-47026], [165548], [-139308], [332842], [-8556], [-69554], [529240], [-13040], [-351240], [626128], [-184826], [357522], [-696508], [-68670], [827222], [143382], [542600], [1127756], [-897976], [-467612], [446140], [-701486], [384198], [953780], [569540], [1066642], [1482430], [-215110], [-1931036], [2037562], [639066], [-37744], [-1839210], [1064720], [-1009520], [1449096], [-2723106], [-1100404], [1735118], [-1591446], [2333760], [2810814], [-939310], [-3098512], [-228266], [-1036692], [-211776], [-1413246], [1090336], [-2642526], [-6254268], [-973924], [864618], [3660458], [4936672], [-731304], [-4866198], [-5719866], [-6820342], [-7459338], [-5238484], [8959948], [-1753536], [927568], [-8437326], [1332780], [3644944], [-3261662], [-2187410], [-10389020], [12772402], [-5222384], [5740470], [-2304476], [5121498], [-2386368], [5241512], [9115484], [3050430], [-14340822], [3126616], [-11509848], [-12178198], [-1776286], [5955164], [-14535108], [10734938], [-16281504], [-7919472], [-1344782], [-20223466], [-7178024], [9612170], [-9668154], [-389854], [-3739852], [-24988606], [-9835838], [-21362448], [-6530252], [16657100], [-23953592], [7489818], [-4718448], [-26739710], [-14433070], [8555104], [19227150], [25331612], [-31316178], [-484890], [5326228], [-32177714], [-8074076], [8049222], [13588940], [-3677208], [32537872], [11272338], [7696944], [4587846], [-11466558], [-30254986], [9830524], [23227248], [30550060], [-22150246], [-12672264], [-40284030], [-13410432], [-27321270], [-8717450], [16298358], [-5490668], [45167540], [23801050], [9364784], [-43391584], [-4352936], [37482718], [5923086], [21514302], [15261360], [46742250], [64196730], [32276792], [47669110], [51705824], [10127850], [-59228300], [-32384344], [-36901116], [-21132332], [56792982], [-9862050], [-19999970], [-29810978], [33591350], [-50560956], [18968842], [-8899160], [-63054644], [-42823282], [-7989244], [71972478], [-25136122], [90612638], [55201210], [-25835282], [-69328726], [49943376], [86653126], [37137568], [13069924], [52356196], [-38986452], [-48568650], [75196976], [99046062], [11864318], [72566848], [19568862], [-104222526], [23046736], [-69115428], [-24170344], [-27847352], [28196416], [52749270], [-37106778], [118327018], [16248898], [100513980], [91787718], [113664560], [-24338196], [134499350], [-53950078], [-87796882], [68310084], [42027066], [-49671558], [-69486616], [-54834080], [-63575090], [-38677728], [77934368], [104546876], [-3418762], [109422890], [-89074960], [-37039558], [27843956], [-96489032], [-34442590], [143832330], [-40347624], [141853604], [-153939036], [-99998576], [-5415826], [30422982], [34030886], [-171514998], [-169752340], [-71824986], [-75105962], [-137152570], [80378192], [11239134], [-53002520], [-207372484], [-219250246], [209813620], [-75743880], [-192890514], [-13401500], [107766558], [167737950], [-68256640], [-83614630], [-214691448], [-142667012], [-131423108], [-12484376], [-99850028], [188602828], [119036902], [133247026], [-45928178], [156520836], [258396520], [-144117796], [-109726976], [40809448], [-46243516], [-113665638], [-36933310], [70173630], [-59599540], [123757146], [193893222], [-103905292], [122793906], [29477276], [-118150722], [111481198], [31313648], [-32687158], [-208024596], [209623796], [242393264], [-240180894], [-236429732], [146472774], [-41310912], [-92088898], [204317938], [110720836], [336209010], [-163008902], [-172285656], [238374594], [-267539694], [234069620], [-356929642], [12098332], [-332476564], [-298936370], [-316895044], [364929050], [41277448], [51041282], [-393060642], [236572096], [226657006], [-101403252], [-242449344], [-160984008], [-294466062], [25761682], [166488134], [131274690], [388708764], [265870060], [130087280], [-246563420], [416654818], [-149161958], [328725412], [-255858660], [355026998], [-91116150], [-303318998], [-125321112], [-465998030], [-311707986], [440615244], [-379565468], [303259776], [246683286], [175753104], [529080518], [514106614], [242810608], [-50484922], [504023366], [286948080], [-223918540], [-252050434], [-526085006], [-28460170], [141180450], [-55439478], [-252001210], [295972964], [-156739758], [-227078502], [249609332], [487000766], [75456546], [-36865496], [-101823038], [164590774], [166038020], [-272791982], [-79540588], [-43837330], [-627127880], [-626472730], [-334706476], [-595668834], [554661126], [85961832], [533876350], [-281090132], [72892512], [-376316520], [659391468], [-497778304], [247611744], [49133330], [-320259950], [-479501186], [446968040], [-152584870], [373594246], [-60041880], [-270296082], [-1622800], [39083922], [-202850288], [-191244994], [-117094252], [-752344526], [-471780078], [-204969608], [-521041502], [-509557530], [110551060], [513572988], [829351912], [-462025662], [97334578], [780837700], [220940202], [-396698574], [119629062], [386436932], [-430328162], [-760447992], [-336539438], [752838608], [-650442918], [-26249844], [141937040], [-402406898], [-292086], [-16926846], [582324716], [-241665280]]
